# ============================================================================
# Replicating  Chen, Pan & Xu (2016)  
# ============================================================================

# Function to install missing packages
install_all <- function(packages) {
  installed_pkgs <- rownames(installed.packages())
  for (pkg in packages) {
    if (!pkg %in% installed_pkgs) {
      install.packages(pkg, repos = 'http://cran.us.r-project.org')
    }
  }
}

# List of packages to be installed
packages <- c(
  "haven", "tidyr", "dplyr", "knitr", 
  "kableExtra", "ggplot2", "sandwich", 
  "estimatr", "modelsummary", "janitor"
)

# Run the function
install_all(packages)

# Load all packages without output
invisible(lapply(packages, function(pkg) 
  suppressPackageStartupMessages(require(pkg, character.only = TRUE))))



library(tidyr)
library(dplyr)
library(knitr) 
library(kableExtra)
library(ggplot2)
library(sandwich)
library(estimatr)
library(modelsummary)
library(janitor)
library(haven)

data <- read_dta("forum.dta")


## ----warning = FALSE-----------------------------------------------------------
# Define the covariates, see codebook for further explanation. 
covar <- c("logpop", "logpop00", "popgrow", "sexratio", "logpopden", "migrant", "nonagrhh", "urban", "eduyr", "illit", "minor", "unemploy", "wk_agr", "wk_ind", "wk_ser", "income", "logincome", "loggdp", "avggrowth", "logoutput_agr", "logoutput_ind", "logoutput_ser", "numlfirm", "loginvest", "logsaving", "loggov_rev", "loggov_exp")

variable_names <- c(
  logpop = "Log population",
  logpop00 = "Log population (2000)",
  popgrow = "Population growth (2000-10%)",
  sexratio = "Gender ratio (female = 1.00)",
  logpopden = "Log population density (person/km2)",
  migrant = "Migrant share of population (%)",
  nonagrhh = "Non-agriculture household share of population (%)",
  urban = "Permanent urban residents share of population (%)",
  eduyr = "Average years of education",
  illit = "Illiteracy rate among population age above 15 (%)",
  minor = "Ethnic minority share of population (%)",
  unemploy = "Unemployment rate (%)",
  wk_agr = "Work force in agriculture (%)",
  wk_ind = "Work force in industry (%)",
  wk_ser = "Work force in services (%)",
  income = "GDP per capita (1,000 CNY)",
  logincome = "Log GDP per capita",
  loggdp = "Log GDP",
  avggrowth = "Average nominal GDP growth (2000-10)",
  logoutput_agr = "Log agricultural output",
  logoutput_ind = "Log industrial output",
  logoutput_ser = "Log services output",
  numlfirm = "Number of enterprises above designated size (annual sales above 5 million CNY)",
  loginvest = "Log total investment from households, enterprises, and government",
  logsaving = "Log total saving (total outstanding bank deposits of rural and urban households)",
  loggov_rev = "Log total government revenue",
  loggov_exp = "Log total government expenditure"
)


# Function to generate summary statistics
func_gen_summary_stats <- function(data, var, variable) {
  data %>%
    summarise(
      variable = variable,
      count = sum(!is.na(.data[[var]])),
      mean = round(mean(.data[[var]], na.rm = TRUE), 2),
      sd = round(sd(.data[[var]], na.rm = TRUE), 2),
      min = round(min(.data[[var]], na.rm = TRUE), 2),
      max = round(max(.data[[var]], na.rm = TRUE), 2),
      median = round(median(.data[[var]], na.rm = TRUE), 2)
    ) %>%
    as.data.frame()
}

# Generate summary statistics for each variable
summary_stats <- do.call(rbind, Map(function(var, variable) func_gen_summary_stats(data, var, variable), covar, variable_names))
rownames(summary_stats) <- NULL  

kable(summary_stats, format = "html") %>%
  kable_styling(full_width = F, bootstrap_options = c("striped", "hover", "condensed"))



## ------------------------------------------------------------------------------
# Calculate the mean of each covariate for the 4 experimental groups
standardized_data <- data %>%
  mutate(across(all_of(covar), ~ (. - mean(.)) / sd(.)))

# The following variables are dropped because they are either: 
# 1) logged values, or 
# 2) proportions

# small_vars <- c("logpop", "logpop00", "logpopden", "logincome", "loggdp", "logoutput_agr", "logoutput_ind", "logoutput_ser", "loginvest", "logsaving", "loggov_rev", "loggov_exp", "income", "numlfirm","avggrowth")

non_logged_vars <- c("popgrow", "sexratio", "migrant", "nonagrhh", "urban","eduyr", "illit", "minor", "unemploy", "wk_agr","wk_ind", "wk_ser")

names_non<- c(
  "Population growth (2000-10%)",
  "Gender ratio (female = 1.00)",
  "Migrant share of population (%)",
  "Non-agriculture household share of population (%)",
  "Permanent urban residents share of population (%)",
  "Average years of education",
  "Illiteracy rate among population age above 15 (%)",
  "Ethnic minority share of population (%)",
  "Unemployment rate (%)",
  "Workforce in agriculture (%)",
  "Workforce in industry (%)",
  "Workforce in services (%)"
)


# Summarize means for each treatment group
means <- data %>%
  select(all_of(non_logged_vars), treat) %>%
  mutate(across(all_of(non_logged_vars), ~ (. - mean(.)) / sd(.))) %>%
  group_by(treat) %>%
  summarize(across(all_of(non_logged_vars), mean, na.rm = TRUE))

# Reshape data for plotting, keeping only non-logged covariates
means_long <- means %>%
  pivot_longer(cols = -treat, names_to = "Covariate", values_to = "Mean") %>%
  mutate(Covariate = factor(Covariate, levels = non_logged_vars, labels = names_non))

# Define unique symbols and colors for the 4 groups
symbols <- c(16, 17, 22, 23)

# Create the plot with adjusted legend position, layout, and colors
ggplot(means_long, aes(x = Mean, y = Covariate, shape = as.factor(treat))) +
  geom_point(size = 3) +
  scale_shape_manual(values = symbols,
                     labels = c("Control", 
                                "Collective Action Threat", 
                                "Tattling Threat", 
                                "Claims of Loyalty")) +
  labs(title = "Covariate Balance",
       x = "Standardized Mean Differences",
       y = "") +
  theme_minimal() +
  theme(legend.position = "bottom",
        legend.box = "horizontal",
        legend.title = element_blank(),
        legend.text = element_text(size = 8),
        legend.key.size = unit(1, "lines"),
        legend.spacing = unit(0.2, "lines"),
        legend.margin = margin(0, 0, 0, 0)) +
  guides(shape = guide_legend(ncol = 2)) +
  scale_x_continuous(limits = c(-1, 1))


## ------------------------------------------------------------------------------
# Regression Analysis for Tables
vars_to_demean <- c("forum_response", "forum_recent_response", 
                    "forum_imme_viewable", "post_email", "post_name",
                    "post_idnum", "post_tel", "post_address",
                    "logpop", "nonagrhh", "urban", "eduyr",
                    "unemploy", "minor")

# Function to demean variables
demean_vars <- function(data, vars) {
  data %>%
    mutate(across(all_of(vars), 
                  list(dm = ~ . - mean(. , na.rm = TRUE)),
                  .names = "dm_{.col}"))
}
data <- demean_vars(data, vars_to_demean)

# Control_dem: Interaction between treat and demeaned demographic variables
control_dem <- "(dm_logpop + dm_nonagrhh + dm_urban + dm_eduyr + dm_unemploy + dm_minor)"
# Control_web: Interaction between treat and demeaned web-related variables
control_web <- "(dm_forum_response + dm_forum_recent_response + dm_forum_imme_viewable + dm_post_email + dm_post_name + dm_post_idnum + dm_post_tel + dm_post_address)"

# -----------------------------
# Table 2 
# -----------------------------

# Table 2 Regressions
# -----------------------------
# 2(a). Conditional Regressions (if posted)
# -----------------------------
# a. Basic OLS regression without fixed effects
model_uncond_ols <- lm_robust(response ~ tr1 + tr2 + tr3, clusters = prov, data = data)
# b. Fixed effects regression with city fixed effects
model_uncond_fe <- lm_robust(response ~ tr1 + tr2 + tr3, clusters = prov, fixed_effects = ~city, data = data)
# c. Fixed effects with demographic controls
model_uncond_fe_dem <- lm_robust(as.formula(paste("response ~ tr1 + tr2 + tr3 +", control_dem)), fixed_effects = ~city,clusters = prov, data = data)

# -----------------------------
# 2(b). Conditional Regressions (if posted)
# -----------------------------
# Subset data where posted == 1
forum_subset <- subset(data, posted == 1)

# a. Basic OLS regression on posted data
model_cond_ols <- lm_robust(response ~ tr1 + tr2 + tr3, clusters = prov, data = forum_subset)
# b. Fixed effects regression with city fixed effects on posted data
model_cond_fe <- lm_robust(response ~ tr1 + tr2 + tr3, clusters = prov, fixed_effects = ~city, data = forum_subset)
model_cond_c1 <- lm_robust(as.formula(paste("response ~ tr1 + tr2 + tr3 +", control_dem)), fixed_effects = ~city,clusters = prov, data = forum_subset)
# c. Fixed effects with demographic and web controls on posted data
model_cond_fe_controls <- lm_robust(as.formula(paste("response ~ tr1 + tr2 + tr3 +", control_dem, "+", control_web)), clusters = prov, fixed_effects = ~city, data = forum_subset)

# -----------------------------
# Table 3 
# -----------------------------
# Table 3 Regressions
# a. Basic OLS regression for response_public
model_public_uncond_ols_2 <- lm_robust(response_public ~ tr1 + tr2 + tr3, clusters = prov, data = data)
# Fixed effects regression with city fixed effects and demographic controls
model_public_uncond_fe_dem_2 <- lm_robust(as.formula(paste("response_public ~ tr1 + tr2 + tr3 +", control_dem)), clusters = prov, fixed_effects = ~city, data = data)

# b. Conditional regressions for response_public if posted
# Basic OLS on posted data
model_public_cond_ols_2 <- lm_robust(response_public ~ tr1 + tr2 + tr3, clusters = prov, data = forum_subset)
# Fixed effects with controls on posted data
model_public_cond_fe_controls_2 <- lm_robust(as.formula(paste("response_public ~ tr1 + tr2 + tr3 +", control_dem, "+", control_web)), clusters = prov, fixed_effects = ~city, data = forum_subset)


models_2 <- list(
  "Unconditional (1)" = model_public_uncond_ols_2, 
  "Unconditional (2)" = model_public_uncond_fe_dem_2, 
  "Conditional (3)" = model_public_cond_ols_2, 
  "Conditional (4)" = model_public_cond_fe_controls_2
)


# Regression Analysis for plotting
outcome_vars <- c("attempted", "posted", "forum_response", "forum_recent_response", "forum_imme_viewable", "post_name", "post_tel", "post_email", "post_address", "post_idnum")

# -----------------------------
# Figure 1 & 2
# -----------------------------
results <- outcome_vars %>%
  lapply(function(var) {
    lm_model <- lm_robust(as.formula(paste(var, "~ tr1 + tr2 + tr3")), data = data)
    lm_summary <- summary(lm_model)
    
    # Extract coefficients and standard errors
    intercept <- coef(lm_model)["(Intercept)"]
    tr1_coef <- intercept + coef(lm_model)["tr1"]
    tr2_coef <- intercept + coef(lm_model)["tr2"]
    tr3_coef <- intercept + coef(lm_model)["tr3"]
    
    # Calculate modified estimates and confidence intervals
    estimates <- c(intercept, tr1_coef, tr2_coef, tr3_coef)
    se <- c(
      lm_summary$coefficients["(Intercept)", "Std. Error"],
      lm_summary$coefficients["tr1", "Std. Error"],
      lm_summary$coefficients["tr2", "Std. Error"],
      lm_summary$coefficients["tr3", "Std. Error"]
    )
    
    lower <- estimates - 1.96 * se
    upper <- estimates + 1.96 * se
    
    # Create a data frame of results for this outcome variable
    data.frame(
      outcome = var,
      treatment = c("Control", "T1", "T2", "T3"),
      estimate = estimates,
      se = se,
      lower = lower,
      upper = upper
    )
  }) %>%
  bind_rows()

results <- results %>%
  mutate(
    outcome = recode(outcome,
                     "attempted" = "Posting attempt made",
                     "posted" = "Request successfully posted",
                     "post_address" = "Requiring address",
                     "post_email" = "Requiring email",
                     "post_idnum" = "Requiring ID number",
                     "post_tel" = "Requiring telephone contact",
                     "post_name" = "Requiring name"),
    treatment = factor(treatment, levels = c("Control", "T1", "T2", "T3"),labels = c("Control", "Collective Action Threat", "Tattling Threat", "Claims of Loyalty"))
  )


# -----------------------------
# Figure 3 & 5
# -----------------------------
main_vars <- c("response","response_public", "response_content", "response_time")


results_main <- main_vars %>%
  lapply(function(var) {
    # Fit the robust linear model
    lm_model <- lm_robust(as.formula(paste(var, "~ tr1 + tr2 + tr3")), data = forum_subset)
    lm_summary <- summary(lm_model)
    
    # Extract coefficients
    intercept <- coef(lm_model)["(Intercept)"]
    tr1_coef <- intercept + coef(lm_model)["tr1"]
    tr2_coef <- intercept + coef(lm_model)["tr2"]
    tr3_coef <- intercept + coef(lm_model)["tr3"]
    
    # Calculate the adjusted estimates
    estimates <- c(intercept, tr1_coef, tr2_coef, tr3_coef)
    se <- c(
      lm_summary$coefficients["(Intercept)", "Std. Error"],
      lm_summary$coefficients["tr1", "Std. Error"],
      lm_summary$coefficients["tr2", "Std. Error"],
      lm_summary$coefficients["tr3", "Std. Error"]
    )
    
    # Calculate the adjusted confidence intervals
    lower <- estimates - 1.96 * se
    upper <- estimates + 1.96 * se
    
    # Create a data frame of results for this outcome variable
    data.frame(
      outcome = var,
      treatment = c("Control", "Collective Action", "Tattling Threat", "Claims of Loyalty"),
      estimate = estimates,
      se = se,
      lower = lower,
      upper = upper
    )
  }) %>%
  bind_rows()

results_main <- results_main %>%
  mutate(
    outcome = recode(outcome,
      "response" = "Government response received", 
      "response_public" = "Government response received and publicly viewable", 
      "response_content" = "Government response content", 
      "response_time" = "Government response time"
    )
  )


## ------------------------------------------------------------------------------
ggplot(results %>% filter(outcome %in% c("Posting attempt made", "Request successfully posted")), 
       aes(x = treatment, y = estimate, ymin = lower, ymax = upper, color = outcome)) +
  geom_point() +
  geom_errorbar(width = 0.2) +
  facet_wrap(~ outcome, scales = "fixed") +
  ylim(0, 1) +  
  labs(title = "Availability and Openness of County Government Web Forums",
       x = "Treatment Group", y = "Estimate") +
  theme_minimal() +
  theme(legend.position = "none",
        axis.text.x = element_text(angle = 45, hjust = 1))


## ------------------------------------------------------------------------------
ggplot(results %>% filter(outcome %in% c("Requiring name", "Requiring telephone contact", "Requiring email", "Requiring address", "Requiring ID number")), 
       aes(x = treatment, y = estimate, ymin = lower, ymax = upper, color = outcome)) +
  geom_point() +
  geom_errorbar(width = 0.2) +
  facet_wrap(~ outcome, scales = "fixed") + 
  ylim(0, 1) +
  labs(title = "Requirements for Posting to County Government Web Forums",
       x = "Treatment Group", y = "Estimate") +
  theme_minimal() +
  theme(legend.position = "none",
        axis.text.x = element_text(angle = 45, hjust = 1))


## ------------------------------------------------------------------------------
models <- list(
  "Unconditional" = model_uncond_ols, 
  "Unconditional FE" = model_uncond_fe, 
  "Unconditional FE&Control" = model_uncond_fe_dem, 
  "Conditional" = model_cond_ols, 
  "Conditional FE" = model_cond_fe, 
  "Conditional FE&Control" = model_cond_fe_controls
)
coef_map <- c(
  `(Intercept)` = "Constant",
  `tr1` = "T1: Collective Action Threat", 
  `tr2` = "T2: Tattling Threat", 
  `tr3` = "T3: Claims of Loyalty"
)

mo1 <- modelsummary(
  models,
  coef_map = coef_map,
  statistic = "({std.error}{stars})",
  gof_omit = "IC|LL|AIC|BIC|Log",
  coef_omit = "Intercept",
  fmt = 3,
  output = "kableExtra"  
)

mo1 %>%
  kable_styling(full_width = F, bootstrap_options = c("striped", "hover", "condensed"))


## ------------------------------------------------------------------------------
models_2 <- list(
  "Unconditional" = model_public_uncond_ols_2, 
  "Unconditional FE&Control" = model_public_uncond_fe_dem_2, 
  "Conditional" = model_public_cond_ols_2, 
  "Conditional FE&Control" = model_public_cond_fe_controls_2
)


mo2 <- modelsummary(
  models_2,
  coef_map = coef_map,
  statistic = "({std.error}{stars})",
  gof_omit = "IC|LL|AIC|BIC|Log",
  coef_omit = "Intercept",
  fmt = 3,
  output = "kableExtra"  
)

mo2 %>%
  kable_styling(full_width = F, bootstrap_options = c("striped", "hover", "condensed"))


## ------------------------------------------------------------------------------
ggplot(results_main %>% filter(outcome %in% c("Government response received", "Government response received and publicly viewable")), 
       aes(x = factor(treatment, levels = c("Control", "Collective Action", "Tattling Threat", "Claims of Loyalty")), 
           y = estimate, ymin = lower, ymax = upper, color = outcome)) +
  geom_point() +
  geom_errorbar(width = 0.2) +
  facet_wrap(~ outcome, scales = "fixed", nrow = 1) + 
  ylim(0, 1) + 
  labs(title = "Causal Effects of Treatments on Responsiveness",
       x = "Treatment Group", y = "Estimate") +
  theme_minimal() +
  theme(legend.position = "none",
        axis.text.x = element_text(angle = 45, hjust = 1))


# -----------------------------
# Table 4
# -----------------------------
# Create a cross-tab with row proportions
cross_tab <- data %>%
  mutate(
    treat = case_when(
      treat == 0 ~ "Control",
      treat == 1 ~ "T1: Collective Action Threat",
      treat == 2 ~ "T2: Tattling Threat",
      treat == 3 ~ "T3: Claims of Loyalty"
    ),
    response_content = case_when(
      response_content == 0 ~ "No Response",
      response_content == 1 ~ "Deferral",
      response_content == 2 ~ "Referral",
      response_content == 3 ~ "Direct Info"
    )
  ) %>%
  tabyl(treat, response_content) %>%
  adorn_percentages("row") %>%
  adorn_pct_formatting(digits = 2) %>%
  adorn_ns()


kable(cross_tab, format = "html") %>%
  kable_styling(full_width = F, bootstrap_options = c("striped", "hover", "condensed"))


## ------------------------------------------------------------------------------
cross_tab_long <- cross_tab %>%
  pivot_longer(
    cols = -treat,
    names_to = "response_content",
    values_to = "percentage"
  ) %>%
  separate(
    percentage,
    into = c("pct", "n"),
    sep = "%",
    convert = TRUE
  ) 

ggplot(cross_tab_long, aes(x = response_content, y = pct / 100, color = treat)) +
  geom_point(size = 3) +
  geom_errorbar(aes(ymin = (pct - 5) / 100, ymax = (pct + 5) / 100), width = 0.2) +
  facet_wrap(~ treat) +
  ylim(0, 1) +  # Set fixed y-axis limits
  labs(
    title = "Causal Effects of Treatments on Reply Content",
    x = "Response Content",
    y = "Causal Effect of Treatments on Reply Content (%)",
    color = "Treatment Group"
  ) +
  theme_minimal() +
  theme(
    plot.title = element_text(hjust = 0.5, size = 16, face = "bold"),
    axis.text.x = element_text(angle = 45, hjust = 1),
    legend.position = "bottom"
  )

